
#include "bf_register.h"
#include "structures.h"

#include "db_private.h"
#include "functions.h"
#include "numbers.h"
#include "list.h"
#include "utils.h"

#include <sys/time.h>
#include <sys/times.h>
#include <sys/resource.h>

static package
bf_clock(Var arglist, Byte next, void *vdata, Objid progr)
{
  /** clock()
      Return a float representing total CPU seconds used by this process.
  */

    static struct rusage   ru;
    Var r;

    getrusage(RUSAGE_SELF, &ru);
    r = new_float((double)ru.ru_utime.tv_sec + ((double)ru.ru_utime.tv_usec / 1000000.00));

    free_var(arglist);
    return make_var_pack(r);
}


static package
bf_microtime(Var arglist, Byte next, void *vdata, Objid progr)
{
  /** microtime()
      Return a float returning canonical time in microseconds.
  */

    static struct timeval  tv;
    static struct timezone tz = {0, 0};
    Var r;

    gettimeofday(&tv, &tz);
    r = new_float((double)tv.tv_sec + ((double)tv.tv_usec / 1000000.00));

    free_var(arglist);
    return make_var_pack(r);
}


static package
bf_find_verb(Var arglist, Byte next, void *vdata, Objid progr)
{

  /** find_verb(OBJ where, STR verbspec[, ANY executable])
      Return {OBJ definer, INT index} if verb exists[ and is executable].
      Raise E_VERBNF if not.
      Raise E_INVARG if where object is invalid.
  */

  Objid         oid         = arglist.v.list[1].v.obj, foid;
  const char   *vname       = arglist.v.list[2].v.str;
  int           check_x_bit = (arglist.v.list[0].v.num >= 3) && is_true(arglist.v.list[3]);
  Var           r;
  Object       *o;
  Verbdef      *v;
  int           i, found = 0;

  if (!valid(oid)) {
    free_var(arglist);
    return make_error_pack(E_INVARG);
  }

  for (o = dbpriv_find_object(foid = oid); o; o = dbpriv_find_object(foid = o->parent)) {
    for (i = 1, v = o->verbdefs; v; v = v->next, i++)
      if (verbcasecmp(v->name, vname) && (!check_x_bit  ||  (v->perms & VF_EXEC))) {
        found = i;
        goto bf_find_verb_done;
      }
  }

  bf_find_verb_done:

  free_var(arglist);

  if (!found)
    return make_error_pack(E_VERBNF);

  r = new_list(2);

  r.v.list[1].type  = TYPE_OBJ;
  r.v.list[1].v.obj = foid;

  r.v.list[2].type  = TYPE_INT;
  r.v.list[2].v.num = found;

  return make_var_pack(r);
}

/** NOTE:

    I didn't write the following two builtins.  I don't know their original
    origin, but they basically just return the results of functions already
    extant in the server. I changed the registered names (used in MOO) to
    suit my own anal tastes.

    -Quinn <quinn@netsville.com>
*/

static package
bf_verbname_match(Var arglist, Byte next, void *vdata, Objid progr)
{
  /** match_verbname(STR pattern, STR string)
      True if string would match a MOO verb with name 'pattern'.
   */

  Var ret;

  ret.type = TYPE_INT;
  ret.v.num = verbcasecmp(arglist.v.list[1].v.str, arglist.v.list[2].v.str);
  free_var(arglist);
  return make_var_pack(ret);
}

static package
bf_strhash(Var arglist, Byte next, void *vdata, Objid progr)
{
  /** string_hash_as_int(STR string)
      Return a (probably unique) integer describing the given string.
   */

  Var ret;

  ret.type = TYPE_INT;
  ret.v.num = str_hash(arglist.v.list[1].v.str);
  free_var(arglist);
  return make_var_pack(ret);
}

void
register_extensions()
{
    register_function("find_verb", 2, 3, bf_find_verb, TYPE_OBJ, TYPE_STR, TYPE_ANY);
    register_function("microtime", 0, 0, bf_microtime);
    register_function("clock", 0, 0, bf_clock);
    register_function("match_verbname", 2, 2, bf_verbname_match, TYPE_STR, TYPE_STR);
    register_function("string_hash_as_int", 1, 1, bf_strhash, TYPE_STR);
}
